@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # ModRana config files handling
@@ -25,7 +24,7 @@ from modules.configobj.configobj import ConfigObj
 CONFIGS = ["map_config.conf", "user_config.conf"]
 
 
-class Configs:
+class Configs(object):
   def __init__(self, modrana):
     self.modrana = modrana
     self.paths = modrana.paths
@@ -33,6 +32,7 @@ class Configs:
     self.userConfig = {}
     self.mapConfig = {}
     self.mapLayers = {}
+    self.mapGroups = {}
 
     # check if config files exist
     self.checkConfigFilesExist()
@@ -103,7 +103,6 @@ class Configs:
     self.loadMapConfig()
     self.loadUserConfig()
 
-
   def getUserConfig(self):
     return self.userConfig
 
@@ -118,18 +117,26 @@ class Configs:
           self.userConfig = config
 
     except Exception, e:
-      print "modRana configs: loading user_config.conf failed"
-      print "modRana configs: check the syntax"
-      print "modRana configs: and if the config file is present in the main directory"
-      print "modRana configs: this happened:\n%s\nconfig: that's all" % e
+      print("modRana configs: loading user_config.conf failed")
+      print("modRana configs: check the syntax")
+      print("modRana configs: and if the config file is present in the main directory")
+      print("modRana configs: this happened:\n%s\nconfig: that's all" % e)
 
 
   def getMapLayers(self):
     """
     get the filtered & tweaked map layer info from the map config
     """
+#    print("MAP LAYERS")
+#    for value in self.mapLayers:
+#      print(value)
+#      print(self.mapLayers[value])
     return self.mapLayers
 
+  def getMapGroups(self):
+    """get the dictionary with map layer groups"""
+    return self.mapGroups
+
   def getMapConfig(self):
     """
     get the "raw" map config
@@ -176,11 +183,13 @@ class Configs:
 
     config = {}
     mapLayers = {}
+    mapGroups = {}
     try:
       config = ConfigObj(mapConfigPath)
       for layerID in config['layers'].keys():
         #layerID = str(layerID) # make hashable
         layer = config['layers'][layerID]
+        tempDict = {}
         if allNeededIn(configVariables.keys(), layer.keys()): # check if all needed keys are available
           tempDict = {}
           for var in configVariables:
@@ -188,15 +197,29 @@ class Configs:
             tempDict[configVariables[var]] = layer[var]
           tempDict['minZoom'] = int(tempDict['minZoom']) # convert strings to integers
           tempDict['maxZoom'] = int(tempDict['maxZoom'])
+          tempDict['group'] = layer.get('group', None)
+          tempDict['icon'] = layer.get('icon', "generic")
+
         else:
           print("modRana configs: layer %s is badly defined/formatted" % layerID)
 
         mapLayers[layerID] = tempDict
 
+      # load groups
+      mapGroups = config.get('groups', {})
+
     except Exception, e:
       print("modRana configs: loading map_config.conf failed: %s" % e)
       return False
 
     self.mapConfig = config
     self.mapLayers = mapLayers
-    return True
\ No newline at end of file
+    self.mapGroups = mapGroups
+    return True
+
+  def getUserAgent(self):
+    """return the default modRana User-Agent"""
+    #debugging:
+    # return "Mozilla/5.0 (compatible; MSIE 5.5; Linux)"
+    #TODO: setting from configuration file, CLI & interface
+    return "modRana flexible GPS navigation system (compatible; Linux)"
\ No newline at end of file
