@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # ModRana startup handling
@@ -38,7 +37,8 @@ CURRENT_POSITION_UNKNOWN_ERROR = 7
 
 USE_LAST_KNOWN_POSITION_KEYWORD = "LAST_KNOWN_POSITION"
 
-class Startup:
+
+class Startup(object):
   def __init__(self, modrana):
     self.modrana = modrana
     self.originalStdout = None
@@ -48,15 +48,15 @@ class Startup:
       '-d', metavar="device ID", type=str,
       help="specify device type",
       default=None, action="store",
-      choices=["neo", "pc", "n900", "n9", "q7", "android_chroot"]
+      choices=self.modrana._listAvailableDeviceModulesByID()
     )
     # GUI
     parser.add_argument(
       '-u', metavar="GUI ID", type=str,
       help='specify user interface type (GTK or QML)',
       default=None,
-      action="store",
-      choices=["GTK", "QML"]
+      action="store"
+      #      choices=["GTK", "QML"]
     )
     # local search
     parser.add_argument(
@@ -73,7 +73,7 @@ class Startup:
            ' geographic coordinates with the geo: prefix are supported;'
            ' use "%s" to use last known position '
            'EXAMPLE: "London" or "geo:50.083333,14.416667" or "%s"' % (
-      USE_LAST_KNOWN_POSITION_KEYWORD, USE_LAST_KNOWN_POSITION_KEYWORD)
+             USE_LAST_KNOWN_POSITION_KEYWORD, USE_LAST_KNOWN_POSITION_KEYWORD)
       ,
       default=None,
       action="store"
@@ -234,6 +234,13 @@ class Startup:
       print("startup: parsing coordinates for the --focus-on-coordinates option failed")
       print(e)
 
+  def _getLocalSearchLocation(self):
+    location = self.args.local_search_location
+    if location is not None:
+      return location.strip() # remove leading & trailing whitespace
+    else:
+      return location
+
   def _earlyLocalSearch(self):
     """handle CLI initiated local search that returns a static map URL"""
     """for local search, we need to know our position, so we need at least the
@@ -250,9 +257,9 @@ class Startup:
     # now check if a location for the local search was provided from CLI or if we need to find our location
     # using GPS or equivalent
 
-    if self.args.local_search_location is not None:
+    location = self._getLocalSearchLocation()
+    if location is not None:
       # we use the location provided from CLI, no need to load & start location
-      location = self.args.local_search_location
       if self._useLastKnownPos(location):
         pos = self._getCurrentPosition(loadLocationModule=True, useLastKnown=True)
         if pos is None:
@@ -395,47 +402,45 @@ class Startup:
     if m:
       m.sendMessage(message)
 
-  def _fixCB(self, key, newValue, oldValue, startTimestamp, main):
+  def _fixCB(self, key, newValue, oldValue, startTimestamp, location):
+    """checks for fix and terminates the location & mainloop once
+    either a valid fix is established or once the timeout is reached"""
     print('fix value: %d' % newValue)
     stop = False
     # wait for 3D lock for up to 30 seconds
     if time.time() - startTimestamp > 30:
+      print('fix timed out')
       stop = True
     elif newValue == 3: # 3 = 3D lock
+      print("3D fix established")
       stop = True
     if stop:
-      # quite the main loop so that _getCurrentPosition can finish
-      main.quit()
+    # quite the main loop so that _getCurrentPosition can finish
+    #      main.quit()
+      location.stopLocation()
 
   def _getCurrentPosition(self, loadLocationModule=False, useLastKnown=False):
     """get current position on a system in early startup state"""
 
-    # liblocation needs the main loop to work properly
-    import gobject
-
-    main = gobject.MainLoop()
-
-    # register fix CB
-    self.modrana.watch('fix', self._fixCB, time.time(), main)
-
     # do we need to load the location module ?
-    # we usually need to if we handle an early task that happens before regular startup
+    # we usually need to load it if we handle an early task that happens before regular startup
     if loadLocationModule:
       # the location module might need the device module to handle location on some devices
       self.modrana._loadDeviceModule()
       # load the location module
       l = self.modrana._loadModule("mod_location", "location")
-      print('startup: N900 location module loaded')
+      # register fix CB
+      self.modrana.watch('fix', self._fixCB, [time.time(), l])
+      print('startup: location module loaded')
       # start location
-      l.startLocation()
-      print('startup: N900 location started')
+      l.startLocation(startMainLoop=True)
+      print('startup: location started')
     else:
       l = self.modrana.m.get("location", None)
 
-    # TODO: Qt support (blame the #mccXII) :)
-    # the fix watch will kill the main loop once a 3D fix is established
-    # or once it times out
-    main.run()
+    # the fix watch will kill the main loop once a 3D fix is
+    # established or once it times out,
+    # then the rest of the code in this function will be executed
 
     fix = self.modrana.get('fix', None)
     if fix in (2, 3):
@@ -453,15 +458,15 @@ class Startup:
     #          # GPSD and liblocation need a nudge
     #          # to update the fix when the GUI mainloop is not running
     #          #self.modrana.dmod._libLocationUpdateCB()
-    #          print self.modrana.dmod.lDevice.online
-    #          print self.modrana.dmod.lDevice.status
-    #          print self.modrana.dmod.lDevice.satellites_in_view
-    #          print self.modrana.dmod.lDevice.fix
+    #          print(self.modrana.dmod.lDevice.online)
+    #          print(self.modrana.dmod.lDevice.status)
+    #          print(self.modrana.dmod.lDevice.satellites_in_view)
+    #          print(self.modrana.dmod.lDevice.fix)
     #        if l.provider:
     #          pos = l.provider.getFix().position
     #        else:
     #          pos = self.modrana.get('pos', None)
-    #        print pos
+    #        print(pos)
     #        if pos is not None:
     #          break
     #
@@ -474,9 +479,9 @@ class Startup:
 
     if pos is None: # as a last resort, try last known position, if available
       if not useLastKnown:
-        print "startup: current position unknown"
+        print("startup: current position unknown")
       else:
-        print "startup: using last known position"
+        print("startup: using last known position")
         # we might need to load options "manually" if run early
         if not self.modrana.optLoadingOK:
           self._loadOptions()
