@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # A Neo FreeRunner modRana device-specific module.
@@ -20,16 +19,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-from base_device_module import deviceModule
+from base_device_module import DeviceModule
 
 def getModule(m,d,i):
   return(AndroidChrot(m,d,i))
 
-class AndroidChroot(deviceModule):
+class AndroidChroot(DeviceModule):
   """A modRana device-specific module for Android chroot"""
   
   def __init__(self, m, d, i):
-    deviceModule.__init__(self, m, d, i)
+    DeviceModule.__init__(self, m, d, i)
     self.tempUnfullscreen = False
 
   def getDeviceIDString(self):
@@ -78,11 +77,4 @@ class AndroidChroot(deviceModule):
   def needsQuitButton(self):
     """due to no window decoration, own quit button
     might be needed"""
-    return True
-
-
-if(__name__ == "__main__"):
-  a = device_example({}, {})
-  a.update()
-  a.update()
-  a.update()
+    return True
\ No newline at end of file
