@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 #----------------------------------------------------------------------------
 # Display map tile images (+ position cursor)
@@ -19,24 +18,29 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
 from __future__ import with_statement # for python 2.5
-import urllib
-from base_module import ranaModule
+import StringIO
+from modules.base_module import RanaModule
 from threading import Thread
 import threading
 import os
 import traceback
 import urllib2
 import time
-import modrana_utils
+from core import utils
 from modules import urllib3
-import rectangles
+from core import rectangles
 
-from tilenames import *
+from core.tilenames import *
 
 import socket
 #timeout = 30 # this sets timeout for all sockets
 #socket.setdefaulttimeout(timeout)
 
+
+# if image manipulation tools are available, import them
+# and otherwise disable tile image manipulation
+IMAGE_MANIPULATION_IMPORT_SUCCESS = False
+
 # only import GKT libs if GTK GUI is used
 from core import gs
 
@@ -45,15 +49,26 @@ if gs.GUIString == "GTK":
   import gobject
   import cairo
 
+  # as the image manipulation is dependent on GTK being
+  # used, only load it if using the GTK GUI
+  try:
+    import Image
+    import ImageOps
+    IMAGE_MANIPULATION_IMPORT_SUCCESS = True
+  except ImportError, e:
+    print('mapTiles: import of image manipulation tools unsuccessful'
+          ' - tile image manipulation disabled')
+    print(e)
+
 def getModule(m, d, i):
-  return(MapTiles(m, d, i))
+  return MapTiles(m, d, i)
 
 
-class MapTiles(ranaModule):
+class MapTiles(RanaModule):
   """Display map images"""
 
   def __init__(self, m, d, i):
-    ranaModule.__init__(self, m, d, i)
+    RanaModule.__init__(self, m, d, i)
     self.images = [{}, {}] # the first dict contains normal image data, the second contains special tiles
     self.imagesLock = threading.RLock()
     self.threads = {}
@@ -68,7 +83,7 @@ class MapTiles(ranaModule):
               platform dependent value,
               user configurable
     """
-    self.loadRequestCStack = modrana_utils.SynchronizedCircularStack(self.loadRequestCStackSize)
+    self.loadRequestCStack = utils.SynchronizedCircularStack(self.loadRequestCStackSize)
     #    self.loadingNotifyQueue = Queue.Queue(1)
     self.downloadRequestPool = []
     self.downloadRequestPoolLock = threading.Lock()
@@ -117,17 +132,20 @@ class MapTiles(ranaModule):
     self.modrana.watch('z', self._updateScalingCB)
     self._storeTiles = self.m.get('storeTiles', None) # get the tile storage module
 
+    # map tile filtering
+    self.modrana.watch('currentTheme', self._updateTileFilteringCB, runNow=True)
+    self.modrana.watch('invertMapTiles', self._updateTileFilteringCB, runNow=True)
+    # check if tile filtering is enabled or should be enabled with current theme
+
   def getTile(self, layer, z, x, y):
-    """
-    return a tile specified by layerID, z, x & y
+    """return a tile specified by layerID, z, x & y
     * first look if such a tile is available from storage
-    * if not, download it
-    """
+    * if not, download it"""
     # check if the tile is in the recently-downloaded cache
     name = self.getTileName(layer, z, x, y)
     cacheItem = self.images[0].get(name, None)
     if cacheItem:
-    #      print "got tile FROM memory CACHE"
+    #      print("got tile FROM memory CACHE")
       return cacheItem[0]
 
     # get layer info
@@ -152,7 +170,7 @@ class MapTiles(ranaModule):
 
     if tileData:
       # check if the data is actually an image, and not an error page
-      if modrana_utils.isTheStringAnImage(tileData):
+      if utils.isTheStringAnImage(tileData):
         tileFolder = self.modrana.paths.getMapFolderPath()
         filePath = os.path.join(tileFolder, self.getImagePath(x, y, z, layerPrefix, layerType))
         self._storeTiles.automaticStoreTile(tileData, layerPrefix, z, x, y, layerType, filePath)
@@ -167,16 +185,15 @@ class MapTiles(ranaModule):
       return None
 
   def _getConnPool(self, layer, url):
-    """
-    get a connection pool for the given layer
-    NOTE: connection pools reuse open connections
-    """
+    """get a connection pool for the given layer
+    NOTE: connection pools reuse open connections"""
     pool = self.connPools.get(layer, None)
     if pool:
       return pool
     else: # create pool
       #headers = { 'User-Agent' : "Mozilla/5.0 (compatible; MSIE 5.5; Linux)" }
-      headers = {'User-Agent': "modRana flexible GPS navigation system (compatible; Linux)"}
+      userAgent = self.modrana.configs.getUserAgent()
+      headers = {'User-Agent': userAgent}
       url = self.mapLayers.get(layer)["tiles"]
       newPool = urllib3.connection_from_url(url=url, headers=headers, maxsize=10, timeout=10, block=False)
       self.connPools[layer] = newPool
@@ -186,25 +203,22 @@ class MapTiles(ranaModule):
     """add a download request to the download manager queue
     the download manager will use its download thread pool to process the request
     NOTE: no checking if tile exists in local storage is done past this point,
-    you have to do this beforehand or you might download an already available tile :)
-    """
+    you have to do this beforehand or you might download an already available tile :)"""
 
     #    print("mapTiles: DOWNLOAD queued")
     name = self.getTileName(layer, z, x, y)
     # check if the tile is being downloaded
     if name in self.threads or name in self.images[0]:
       return
-    """ as we check both in thread list and image cache (where download threads deposit tiles) we don't
-    use the corresponding image cache and thread list locks
-    - the window for collision is ver small
-    - possible collisions:
-    -> falsely discarding a request for a thread that actually just errored out
-    -> adding a request for a thread that actually just started (dl manager will kill such requests, np)
-    - so basically nothing serious, also even when ve falsely discard a loading request,
-    it will probably be soon repeated (this is by design to flush the no longer relevant tiles from the
-    download queue)
-    """
-
+    # as we check both in thread list and image cache (where download threads deposit tiles) we don't
+    # use the corresponding image cache and thread list locks
+    # - the window for collision is ver small
+    # - possible collisions:
+    # -> falsely discarding a request for a thread that actually just errored out
+    # -> adding a request for a thread that actually just started (dl manager will kill such requests, np)
+    # - so basically nothing serious, also even when ve falsely discard a loading request,
+    # it will probably be soon repeated (this is by design to flush the no longer relevant tiles from the
+    # download queue)
     layerInfo = self.mapLayers.get(layer, None)
     if layerInfo is None: # is the layer info valid ?
       print("mapTiles: invalid layer")
@@ -219,37 +233,27 @@ class MapTiles(ranaModule):
       with self.threadListCondition:
         timestamp = time.time()
         request = (name, x, y, z, layer, layerPrefix, layerType, filename, timestamp)
-
         with self.downloadRequestPoolLock: # add a download request
           self.downloadRequestPool.append(request)
-
         self.threadListCondition.notifyAll() # wake up the download manager
 
   def tileInProgress(self, layer, z, x, y):
-    """
-    report if tile is being downloaded
-    """
+    """report if tile is being downloaded"""
     name = self.getTileName(layer, z, x, y)
     return name in self.threads
 
   def tileInMemory(self, layer, z, x, y):
-    """
-    report if tile is stored in memory cache
-    """
+    """report if tile is stored in memory cache"""
     name = self.getTileName(layer, z, x, y)
     return name in self.images[0]
 
   def tileInStorage(self, layer, z, x, y):
-    """
-    report if tile is available from storage
-    """
+    """report if tile is available from storage"""
     return self._storeTiles.tileExists2(layer, z, x, y)
 
   def _updateScalingCB(self, key='mapScale', oldValue=1, newValue=1):
-    """
-    as this only needs to be updated once on startup and then only
-    when scaling settings change this callback driven method is used
-    """
+    """as this only needs to be updated once on startup and then only
+    when scaling settings change this callback driven method is used"""
     if key == 'mapScale':
       scale = int(newValue)
     else:
@@ -278,9 +282,7 @@ class MapTiles(ranaModule):
     while True:
       with self.threadListCondition:
         try:
-          """
-          wait for notification that there might be download requests or free download slots
-          """
+          # wait for notification that there might be download requests or free download slots
           self.threadListCondition.wait()
           if self.shutdownAllThreads:
             print("\nmapTiles: automatic tile download management thread shutting down")
@@ -297,7 +299,7 @@ class MapTiles(ranaModule):
                   request = self.downloadRequestPool.pop() # download most recent requests first
                   (name, x, y, z, layer, layerPrefix, layerType, filename, timestamp) = request
                   if name in self.threads.keys():
-                  #                    print "DISCARDED"
+                  #                    print("TILE DISCARDED")
                     continue
                     # start a new thread
                   self.threads[name] = self.TileDownloader(name, x, y, z, layer, layerPrefix, layerType, filename, self)
@@ -310,7 +312,6 @@ class MapTiles(ranaModule):
                         self.images[0][name] = self.downloadingTile
             else:
               # ** all download slots are full **
-
               # remove old requests
               currentTime = time.time()
               cleanPool = []
@@ -324,39 +325,16 @@ class MapTiles(ranaModule):
                 if notOld(currentTime, request):
                   cleanPool.append(request)
                 else:
-                  """this request timed out,
-                     remove its "Waiting..." tile from image cache,
-                     so that is can be retried"""
+                  # this request timed out,
+                  # remove its "Waiting..." tile from image cache,
+                  # so that is can be retried
                   name = request[0]
-                  self.removeImageFromMemmory(name)
+                  self.removeImageFromMemory(name)
               self.downloadRequestPool = cleanPool
         except Exception, e:
           print("exception in tile download manager thread:\n%s" % e)
           #          traceback.print_exc()
 
-          #  def startTileLoadingThread(self):
-          #    """start the loading-request consumer thread"""
-          #    t = Thread(target=self.tileLoader, name='tile loading thread')
-          #    t.setDaemon(True) # we need that the worker dies with the program
-          #    t.start()
-
-          #  def tileLoader(self):
-          #    """this is a tile loading request consumer thread"""
-          #    while True:
-          #      request = self.loadingNotifyQueue.get(block=True) # consume from this queue
-          #      if request == 'load':
-          #        # start processing loading requests from the stack
-          #        while(1):
-          #          (item,valid) = self.loadRequestCStack.popValid()
-          #          if valid:
-          #            (name, x, y, z, layer) = item
-          #            self.loadImage(name, x, y, z, layer)
-          #          else:
-          #            break # the stack is empty
-          #      elif request == 'shutdown':
-          #        print "\nmapTiles: tile loading thread shutting down"
-          #        break
-
   def _startIdleTileLoader(self):
     """add the tile loader as a gobject mainloop idle callback"""
     if not self.idleLoaderActive: # one idle loader is enough
@@ -375,17 +353,15 @@ class MapTiles(ranaModule):
       if valid:
         (name, x, y, z, layer) = item
         self.loadImage(name, x, y, z, layer)
-        #        print "loaded"
+        #        print("loaded")
         return True # don't stop the idle handle
       else:
-      #        print "quiting"
+      #        print("quiting")
         self.idleLoaderActive = False
         return False # the stack is empty, remove this callback
     except Exception, e:
-      """
-      on an error, we need to shut down or else idleLoaderActive might get stuck
-      and no loader will be started
-      """
+      # on an error, we need to shut down or else idleLoaderActive might get stuck
+      # and no loader will be started
       print("mapTiles: exception in idle loader\n", e)
       self.idleLoaderActive = False
       traceback.print_exc()
@@ -417,10 +393,10 @@ class MapTiles(ranaModule):
       #       that would work on GPRS and a fast connection
       #    """
       #    if self.oldZ != z:
-      ##      print "resetting z"
+      ##      print("resetting z")
       #      self.oldZ = z
       #      if len(self.threads) == self.oldThreadCount:
-      #        print "clearing thread set"
+      #        print("clearing thread set")
       #        self.threads = {}
       #        self.oldThreadCount = len(self.threads)
       #    self.oldThreadCount = len(self.threads)
@@ -445,8 +421,8 @@ class MapTiles(ranaModule):
       proj = self.m.get('projection', None)
       drawImage = self.drawImage # method binding to speed back method lookup
       overlay = self.get('overlay', False)
-      """ if overlay is enabled, we use a special naming
-          function in place of the default one"""
+      # if overlay is enabled, we use a special naming
+      # function in place of the default one
 
       singleGetName = self.getTileName
       ratio = self.get('transpRatio', "0.5,1").split(',') # get the transparency ratio
@@ -499,10 +475,6 @@ class MapTiles(ranaModule):
           angle = self.get('bearing', 0.0)
           if angle is None:
             angle = 0.0
-
-          radAngle = radians(angle)
-
-
           # get the rotation angle from the main class (for synchronization purposes)
           radAngle = radians(self.modrana.mapRotationAngle)
 
@@ -513,8 +485,8 @@ class MapTiles(ranaModule):
           (centerX, centerY) = ((sw / 2.0), (sh / 2.0))
           scP = rectangles.Point(centerX, centerY)
 
-          """create a polygon representing the viewport and rotate around
-          current rotation center it to match the rotation and align with screen"""
+          # create a polygon representing the viewport and rotate around
+          # current rotation center it to match the rotation and align with screen
           p1 = rectangles.Point(sx, sy)
           p2 = rectangles.Point(sx + sw, sy)
           p3 = rectangles.Point(sx, sy + sh)
@@ -530,10 +502,10 @@ class MapTiles(ranaModule):
           v4 = rectangles.Vector(*p4.as_tuple())
           polygon = rectangles.Polygon((v1, v2, v3, v4))
 
-          v1 = rectangles.Vector(*p1.as_tuple())
-          v2 = rectangles.Vector(*p2.as_tuple())
-          v3 = rectangles.Vector(*p3.as_tuple())
-          v4 = rectangles.Vector(*p4.as_tuple())
+#          v1 = rectangles.Vector(*p1.as_tuple())
+#          v2 = rectangles.Vector(*p2.as_tuple())
+#          v3 = rectangles.Vector(*p3.as_tuple())
+#          v4 = rectangles.Vector(*p4.as_tuple())
 
           # enlarge the area of possibly visible tiles due to rotation
           add = self.modrana.gui.expandViewportTiles
@@ -569,11 +541,10 @@ class MapTiles(ranaModule):
                     drawImage(cr, tileImage[0], x1, y1, scale)
                   else:
                     if overlay:
-                      """ check if the separate tiles are already cached
-                      and send loading request/-s if not
-                      if both tiles are in the cache, combine them, cache and display the result
-                      and remove the separate tiles from cache
-                      """
+                      # check if the separate tiles are already cached
+                      # and send loading request/-s if not
+                      # if both tiles are in the cache, combine them, cache and display the result
+                      # and remove the separate tiles from cache
                       layerBack = layerInfo[0][0]
                       layerOver = layerInfo[1][0]
                       nameBack = singleGetName(layer1, z, x, y)
@@ -584,8 +555,8 @@ class MapTiles(ranaModule):
                         if backImage[1]['type'] == "normal" and overImage[1]['type'] == "normal":
                           combinedImage = self.combine2Tiles(backImage[0], overImage[0], alphaOver)
                           # remove the separate images from cache
-                          self.removeImageFromMemmory(nameBack)
-                          self.removeImageFromMemmory(nameOver)
+                          self.removeImageFromMemory(nameBack)
+                          self.removeImageFromMemory(nameOver)
                           # cache the combined image
                           self.storeInMemory(combinedImage, name, "composite")
                           # draw the composite image
@@ -602,7 +573,7 @@ class MapTiles(ranaModule):
             gui = self.modrana.gui
             if gui and gui.getIDString() == "GTK":
               if gui.getShowRedrawTime():
-                print "currently visible tiles: %d/%d" % (visibleCounter, wTiles * hTiles)
+                print("currently visible tiles: %d/%d" % (visibleCounter, wTiles * hTiles))
 
                 #            cr.set_source_rgba(0,1,0,0.5)
                 #            cr.move_to(*p1.as_tuple())
@@ -643,11 +614,10 @@ class MapTiles(ranaModule):
                 else:
                   # tile not found im memory cache, do something else
                   if overlay:
-                    """ check if the separate tiles are already cached
-                    and send loading request/-s if not
-                    if both tiles are in the cache, combine them, cache and display the result
-                    and remove the separate tiles from cache
-                    """
+                    # check if the separate tiles are already cached
+                    # and send loading request/-s if not
+                    # if both tiles are in the cache, combine them, cache and display the result
+                    # and remove the separate tiles from cache
                     layerBack = layerInfo[0][0]
                     layerOver = layerInfo[1][0]
                     nameBack = singleGetName(layer1, z, x, y)
@@ -656,14 +626,12 @@ class MapTiles(ranaModule):
                     overImage = self.images[0].get(nameOver)
                     if backImage and overImage: # both images available
                       if backImage[1]['type'] == "normal" and overImage[1]['type'] == "normal":
-                        """
-                        we check the the metadata to filter out the "Downloading..."
-                        special tiles
-                        """
+                        # we check the the metadata to filter out the "Downloading..."
+                        # special tiles
                         combinedImage = self.combine2Tiles(backImage[0], overImage[0], alphaOver)
                         # remove the separate images from cache
-                        self.removeImageFromMemmory(nameBack)
-                        self.removeImageFromMemmory(nameOver)
+                        self.removeImageFromMemory(nameBack)
+                        self.removeImageFromMemory(nameOver)
                         # cache the combined image
                         self.storeInMemory(combinedImage, name, "composite")
                         # draw the composite image
@@ -681,11 +649,12 @@ class MapTiles(ranaModule):
 
       if requests:
         self.loadRequestCStack.batchPush(requests)
-        """can the loadRequestCStack get full ?
-           NO :)
-           it takes the list, reverses it, extends its old internal
-           list with it - from this list a slice conforming to
-           its size limit is taken and set as the new internal list"""
+        # can the loadRequestCStack get full ?
+        # NO :)
+        # it takes the list, reverses it, extends its old internal
+        # list with it - from this list a slice conforming to
+        # its size limit is taken and set as the new internal list
+
         # notify the loading thread using the notify Queue
         #        self.loadingNotifyQueue.put("load", block=False)
         # try to start the idle tile loader
@@ -724,7 +693,7 @@ class MapTiles(ranaModule):
     ct2.paint_with_alpha(alphaOver)
     return backImage
 
-  def removeImageFromMemmory(self, name, dictIndex=0):
+  def removeImageFromMemory(self, name, dictIndex=0):
     # remove an image from memory
     if self.cacheImageSurfaces:
       with self.imagesLock: #make sure no one fiddles with the cache while we are working with it
@@ -793,11 +762,12 @@ class MapTiles(ranaModule):
 
     start1 = time.clock()
     pixbuf = self._storeTiles.getTile(layerPrefix, z, x, y, layerType)
-    """None from getTiles means the tile was not found
-       False means loading the tile from file to pixbuf failed"""
+
+    # None from getTiles means the tile was not found
+    # False means loading the tile from file to pixbuf failed
     if pixbuf:
       start2 = time.clock()
-      self.storeInMemory(self.pixbufToCairoImageSurface(pixbuf), name)
+      self.storeInMemory(self.pixbuf2cairoImageSurface(pixbuf), name)
       if debug:
         storageType = self.get('tileStorageType', 'files')
         sprint("tile loaded from local storage (%s) in %1.2f ms" % (storageType, (1000 * (time.clock() - start1))))
@@ -812,8 +782,9 @@ class MapTiles(ranaModule):
     if self.get('network', 'full') == 'full':
       sprint("automatic tile download enabled - starting download")
       # use threads
-      """the thread list condition is used to signalize to the download manager,
-      that here is a new download request"""
+
+      # the thread list condition is used to signalize to the download manager,
+      # that here is a new download request
       with self.threadListCondition:
         timestamp = time.time()
         request = (name, x, y, z, layer, layerPrefix, layerType, filename, timestamp)
@@ -837,16 +808,16 @@ class MapTiles(ranaModule):
     # Google sat images are 256 by 256 px, we don't need to check the size
     x = 256
     y = 256
-    ''' create a new cairo surface to place the image on '''
+    # create a new cairo surface to place the image on
     surface = cairo.ImageSurface(0, x, y)
-    ''' create a context to the new surface '''
+    # create a context to the new surface
     ct = cairo.Context(surface)
-    ''' create a GDK formatted Cairo context to the new Cairo native context '''
+    # create a GDK formatted Cairo context to the new Cairo native context
     ct2 = gtk.gdk.CairoContext(ct)
-    ''' draw from the pixbuf to the new surface '''
+    # draw from the pixbuf to the new surface
     ct2.set_source_pixbuf(pixbuf, 0, 0)
     ct2.paint()
-    ''' surface now contains the image in a Cairo surface '''
+    # surface now contains the image in a Cairo surface
     self.storeInMemory(surface, name, type, expireTimestamp, dictIndex)
 
   def filePath2Pixbuf(self, filePath):
@@ -869,8 +840,8 @@ class MapTiles(ranaModule):
     with self.imagesLock: #make sure no one fiddles with the cache while we are working with it
       self.images[dictIndex][name] = (surface, metadata) # store the image in memory
 
-      """ check cache size,
-      if there are too many images, delete them """
+      # check cache size,
+      # if there are too many images, delete them
       if len(self.images[0]) > self.maxImagesInMemory:
         self.trimCache()
         # new tile available, make redraw request TODO: what overhead does this create ?
@@ -880,8 +851,7 @@ class MapTiles(ranaModule):
     """redraw the screen when a new tile is available in the cache
        * redraw only when on map screen (menu == None)
        * redraw only on composite tiles when overlay is on"""
-
-    if self.get('tileLoadedRedraw', True) and self.get('menu', None) == None:
+    if self.get('tileLoadedRedraw', True) and self.get('menu', None) is None:
       overlay = self.get('overlay', False)
       if overlay: # only redraw when a composited tile is loaded with overlay on
         if type == "composite":
@@ -897,17 +867,14 @@ class MapTiles(ranaModule):
        so that trim does not run every time an image is added to a full cache
        -> only the normal image cache needs trimming (images[0]),
        as the special image cache (images[1]) is just created once and not updated dynamically
-       NOTE: the storeInMemory method already locked images, so we don't have to
-       """
+       NOTE: the storeInMemory method already locked images, so we don't have to"""
     trimmingAmount = self.imagesTrimmingAmount
     imagesLength = len(self.images[0])
     if trimmingAmount >= imagesLength:
-      """
-      this means that the trimming amount was set higher,
-      than current length of the cache
-      the result is basically flushing the cache every time it fills back
-      well, I don't have an idea why would someone want to do that
-      """
+      # this means that the trimming amount was set higher,
+      # than current length of the cache
+      # the result is basically flushing the cache every time it fills back
+      # well, I don't have an idea why would someone want to do that
       self.images[0] = {}
     else:
       oldestKeys = sorted(self.images[0], key=lambda image: self.images[0][image][1]['addedTimestamp'])[
@@ -915,27 +882,121 @@ class MapTiles(ranaModule):
       for key in oldestKeys:
         del self.images[0][key]
 
-  def pixbufToCairoImageSurface(self, pixbuf):
+  def _clearTileCache(self):
+    """completely clear the in-memory image cache"""
+    print('mapTiles: clearing the in-memory tile cache')
+    self.images[0] = {}
+
+  def pixbuf2cairoImageSurface(self, pixbuf):
     # this solution has been found on:
     # http://www.ossramblings.com/loading_jpg_into_cairo_surface_python
 
-    """Using pixbufs in place of surface_from_png seems to be MUCH faster for JPEGSs and PNGs alike.
-Therefore we use it as default."""
+    # NOTE: Using pixbufs in place of surface_from_png seems to be MUCH faster
+    # for JPEGSs and PNGs alike, therefore we use it as default
+
+    # do any filtering on the pixbuf
+    pixbuf = self._filterTile(pixbuf)
 
     # Tile images are mostly 256 by 256 px, we don't need to check the size
     x = 256
     y = 256
-    ''' create a new cairo surface to place the image on '''
+    # create a new cairo surface to place the image on '''
     surface = cairo.ImageSurface(0, x, y)
-    ''' create a context to the new surface '''
+    # create a context to the new surface
     ct = cairo.Context(surface)
-    ''' create a GDK formatted Cairo context to the new Cairo native context '''
+    # create a GDK formatted Cairo context to the new Cairo native context
     ct2 = gtk.gdk.CairoContext(ct)
-    ''' draw from the pixbuf to the new surface '''
+    # draw from the pixbuf to the new surface
     ct2.set_source_pixbuf(pixbuf, 0, 0)
     ct2.paint()
     return surface
 
+  def _updateTileFilteringCB(self, key='mapScale', oldValue=1, newValue=1):
+    if key == 'invertMapTiles':
+      if newValue == True:
+        self._enableTileFiltering()
+      elif newValue == False:
+        self._disableTileFiltering()
+      elif newValue == "withNightTheme":
+        theme = self.get('currentTheme', 'default')
+        # TODO: get this from theme config
+        if theme == "night":
+          self._enableTileFiltering()
+        else:
+          self._disableTileFiltering()
+    elif key == "currentTheme":
+      invertMapTiles = self.get('invertMapTiles', False)
+      if invertMapTiles == "withNightTheme":
+        # if switching from night to other -> disable
+        if oldValue == "night" and newValue != "night":
+          self._disableTileFiltering()
+        # switching from other to night -> enable
+        elif oldValue != "night" and newValue == "night":
+          self._enableTileFiltering()
+        # else: switching form some other theme to another theme
+        # -> do nothing
+
+  def _enableTileFiltering(self):
+    """assign the real filtering method, if possible
+    and flush the cache afterwards"""
+    if IMAGE_MANIPULATION_IMPORT_SUCCESS:
+      self._filterTile = self._invertPixbuf
+      self._clearTileCache()
+
+  def _disableTileFiltering(self):
+    """assign the NOP filtering method
+    and flush the cache if needed"""
+    if IMAGE_MANIPULATION_IMPORT_SUCCESS:
+      self._filterTile = self._nop
+      # as image manipulation is possible,
+      # a cache flush is needed
+      self._clearTileCache()
+
+  def _filterTile(self, tilePb):
+    """function hook for image tile processing"""
+    return tilePb
+
+  def _nop(self, arg):
+    """a NOP function used for replacing functions that are not
+    currently used for some reason (such as for example _invertPixbuf)"""
+    return arg
+
+  def _invertPixbuf(self, pixbuf):
+    """do image inversion for the given pixbuf
+    - why PIL & Numpy -> combined, they give usable performance
+    (pure python implementation was about 100x times slower
+    10 vs 1000 ms per tile)"""
+
+#    start1 = time.clock()
+    def pixbuf2Image(pb):
+      return Image.fromstring("RGB",(256,256),pb.get_pixels() )
+
+    def image2pixbuf(im):
+      file1 = StringIO.StringIO()
+      im.save(file1, "ppm")
+      contents = file1.getvalue()
+      file1.close()
+      loader = gtk.gdk.PixbufLoader("pnm")
+      loader.write(contents, len(contents))
+      pixbuf = loader.get_pixbuf()
+      loader.close()
+      return pixbuf
+    #  arr = numpy.array(im)
+    #  return gtk.gdk.pixbuf_new_from_array(arr, gtk.gdk.COLORSPACE_RGB, 8)
+    # Why are we using StringIO here ?
+    #
+    # The above mentioned method works just fine with recent GTK2 on PC
+    # but doesn't work with the highly patched and old version of GTK on
+    # Maemo 5 fremantle.
+    # Other than that, even on the N900 StringIO seems to still be really fast
+    # so it is used even on desktop, eliminating the Numpy dependency.
+    # Also "premature optimization" and all that. :)
+
+    image = pixbuf2Image(pixbuf)
+    image = ImageOps.invert(image)
+    return image2pixbuf(image)
+#    print("tile negative in %1.2f ms" % (1000 * (time.clock() - start1)))
+
   def imageName(self, x, y, z, layer):
     """Get a unique name for a tile image 
     (suitable for use as part of filenames, dictionary keys, etc)"""
@@ -982,11 +1043,27 @@ Therefore we use it as default."""
       url = '%s&x=%d&y=%d&z=%d&r=1' % ( # I have no idea what the r parameter is, r=0 or no r => grey square
                                         layerDetails['tiles'],
                                         x, y, z)
+#    elif coords == 'chartbundle': # chartbundle
+##      print("CHARTBUNDLE")
+#      y = ((2 ** (z - 1) - 1) - y)
+#      # TODO: fix projection so this is usable
+#      # NOTE: looks like with the y fix from Yahoo tilenames,
+#      # the map is drawn correctly, but current position is somewhere in Antarctica
+#      # -> thi probably means some projection lat lon -> x y errors for this layer
+#
+#      url = '%s%d/%d/%d.%s' % (
+#      layerDetails['tiles'],
+#      z, x, y,
+#      layerDetails.get('type', 'png'))
     else: # OSM, Open Cycle, T@H -> equivalent to coords == osm
       url = '%s%d/%d/%d.%s' % (
         layerDetails['tiles'],
         z, x, y,
         layerDetails.get('type', 'png'))
+
+#    print("__%s__" % coords)
+#     print("TILE URL")
+#     print(url)
     return url
 
   def shutdown(self):
@@ -1030,38 +1107,34 @@ Therefore we use it as default."""
           self.layer,
           self.filename)
         self.finished = 1
-
       # something is wrong with the server or url
       except urllib2.HTTPError:
         if self.useImageSurface:
           tileDownloadFailedSurface = self.callback.images[1]['tileDownloadFailed'][0]
           expireTimestamp = time.time() + 10
           self.callback.storeInMemory(tileDownloadFailedSurface, self.name, 'semiPermanentError', expireTimestamp)
-          """
-          like this, when tile download fails due to a http error,
-          the error tile is loaded instead
-          like this:
-           - modRana does not immediately try to download a tile that errors out
-           - the error tile is shown without modifying the pipeline too much
-           - modRana will eventually try to download the tile again,
-             after it is flushed with old tiles from the memory
-          """
+          # like this, when tile download fails due to a http error,
+          # the error tile is loaded instead
+          # like this:
+          #  - modRana does not immediately try to download a tile that errors out
+          #  - the error tile is shown without modifying the pipeline too much
+          #  - modRana will eventually try to download the tile again,
+          #    after it is flushed with old tiles from the memory
       except urllib2.URLError:
         if self.useImageSurface:
           tileNetworkErrorSurface = self.callback.images[1]['tileNetworkError'][0]
           expireTimestamp = time.time() + 10
           self.callback.storeInMemory(tileNetworkErrorSurface, self.name, 'error',
             expireTimestamp) # retry after 10 seconds
-          """ as not to DOS the system when we temporarily loose internet connection or other such error occurs,
-               we load a temporary error tile with expiration timestamp instead of the tile image
-               TODO: actually remove tiles according to expiration timestamp :)
-          """
+          # as not to DOS the system when we temporarily loose internet connection or other such error occurs,
+          # we load a temporary error tile with expiration timestamp instead of the tile image
+          # TODO: actually remove tiles according to expiration timestamp :)
 
       # something other is wrong (most probably a corrupted tile)
       except Exception, e:
         self.printErrorMessage(e)
         # remove the status tile
-        self.callback.removeImageFromMemmory(self.name)
+        self.callback.removeImageFromMemory(self.name)
 
       finally: # make really sure we don't get any zombie threads
         self.finished = 1 # finished thread can be removed from the set and retried
@@ -1072,7 +1145,7 @@ Therefore we use it as default."""
         # try to remove its own instance from the thread list, so that the instance could be garbage collected
         if self.name in self.callback.threads.keys():
           del self.callback.threads[self.name]
-          """notify the download manager that a download slot is now free"""
+          # notify the download manager that a download slot is now free
           self.callback.threadListCondition.notifyAll()
 
     def printErrorMessage(self, e):
@@ -1093,12 +1166,9 @@ Therefore we use it as default."""
     def downloadTile(self, name, x, y, z, layer, filename):
       """Downloads an image"""
       url = self.callback.getTileUrl(x, y, z, layer)
-
       request = urllib2.urlopen(url)
-      #      request = urllib.urlopen(url)
       content = request.read()
       request.close()
-
       if self.useImageSurface:
         pl = gtk.gdk.PixbufLoader()
 
@@ -1106,21 +1176,7 @@ Therefore we use it as default."""
 
         pl.close() # this  blocks until the image is completely loaded
         # http://www.ossramblings.com/loading_jpg_into_cairo_surface_python
-        #x = pixbuf.get_width()
-        #y = pixbuf.get_height()
-        # Google sat images are 256 by 256 px, we don't need to check the size
-        x = 256
-        y = 256
-        ''' create a new cairo surface to place the image on '''
-        surface = cairo.ImageSurface(0, x, y)
-        ''' create a context to the new surface '''
-        ct = cairo.Context(surface)
-        ''' create a GDK formatted Cairo context to the new Cairo native context '''
-        ct2 = gtk.gdk.CairoContext(ct)
-        ''' draw from the pixbuf to the new surface '''
-        ct2.set_source_pixbuf(pl.get_pixbuf(), 0, 0)
-        ct2.paint()
-        ''' surface now contains the image in a Cairo surface '''
+        surface = self.callback.pixbuf2cairoImageSurface(pl.get_pixbuf())
         self.callback.storeInMemory(surface, name)
         # like this, corrupted tiles should not get past the pixbuf loader and be stored
       else:
@@ -1134,7 +1190,6 @@ Therefore we use it as default."""
 # modified from: http://www.maptiler.org/google-maps-coordinates-tile-bounds-projection/globalmaptiles.py (GPL)
 def quadTree(tx, ty, zoom ):
   """Converts OSM type tile coordinates to Microsoft QuadTree"""
-
   quadKey = ""
   #		ty = (2**zoom - 1) - ty
   for i in range(zoom, 0, -1):
@@ -1145,5 +1200,4 @@ def quadTree(tx, ty, zoom ):
     if (ty & mask) != 0:
       digit += 2
     quadKey += str(digit)
-
   return quadKey
